Extension { #name : 'Repository' }

{ #category : 'Updating' }
Repository >> at: offset put: aValue [
  "Disallowed, installed after Upgrade2C class >> _patchSecurityPolicy15 has run"
  self shouldNotImplement: #at:put:
]

{ #category : 'Private' }
Repository >> _buildAllRefsResult: allRefsResult forInput: inputArr [

 | resultArr |
 resultArr := Array new: (inputArr size).
 1 to: (allRefsResult size) do: [ :i |  | element |
   element := allRefsResult at: i.
   resultArr at: (inputArr indexOfIdentical: (element at: 1)) put: ((element at: 2) asArray).
 ].
 ^ resultArr.

]

{ #category : 'Private' }
Repository >> _buildAllRefsResult: allRefsResult forInput: inputArr withLimit: aSmallInt [

 | resultArr |
 resultArr := Array new: (inputArr size).
 1 to: (allRefsResult size) do: [ :i |  | element |
   element := allRefsResult at: i.
   resultArr at: (inputArr indexOfIdentical: (element at: 1))
             put: ((element at: 2) enumerateWithLimit: aSmallInt startingAfter: 0)
 ].
 ^ resultArr.

]

{ #category : 'Private' }
Repository >> _buildListInstResult: allInstResult [

 | gsBitmap arrSize element resultArr |

 gsBitmap := GsBitmap newForHiddenSet: #ListInstances.
 gsBitmap removeAll.

 arrSize := allInstResult size.
 arrSize == 1 ifTrue: [
   element := allInstResult at: 1.
   gsBitmap addAll: (element at: 2).
   ^ (element at: 2) size
 ].

 resultArr := Array new.
 1 to: arrSize do: [ :i |
   element := allInstResult at: i.
   resultArr add: (element at: 1).
   resultArr add: ((element at: 2) size).
   gsBitmap addAll: (element at: 2).
 ].
 ^ resultArr.

]

{ #category : 'Private' }
Repository >> _doScan: kind fast: isFast with: inputArgument [

" This method calls does the appropriate tests, and calls the _scan primitive
  which does the bulk of the work.

  If the SessionTemps current at: #GsOverrideNumThreads is defined, then that value is
  used to determine the number of threads to use.

      scanKind                      Function
   ==========================================
   OP_ALL_INSTANCES              allInstances
   OP_ALL_REFERENCES             allReferences
   OP_ALL_REFS_TO_INSTANCES      allReferencesToInstancesOfClasses
   OP_ALL_REFS_BY_PARENT_CLASS   allReferencesByParentClass
   OP_ALL_OBJS_IN_SEC_POLICY     allObjectsInObjectSecurityPolicies
   OP_ALL_OBJS_LARGER_THAN       allObjectsLargerThan

"
| maxThreads percentCpu argClass result inputArg |
inputArg := (inputArgument _isArray and: [inputArgument size > 1])
                ifTrue:[ Array withAll:(IdentitySet withAll: inputArgument)]
                ifFalse:[ inputArgument ].
System needsCommit ifTrue: [ System _error: #rtErrAbortWouldLoseData ] .

isFast ifTrue: [ maxThreads := self _aggressiveMaxThreadCount.
                 percentCpu := 95 ]
       ifFalse: [  maxThreads := self getDefaultNumThreads. percentCpu := 90 ].
result := self _scanWithMaxThreads: maxThreads waitForLock: 60 pageBufSize: 8
               percentCpuActiveLimit: percentCpu scanKind: kind with: inputArg .
argClass := inputArg class.
(kind ~= OP_ALL_REFS_BY_PARENT_CLASS) ifTrue: [
  ((argClass == Array) or: [argClass == GsBitmap]) ifFalse: [
       result := (result at: 1) at: 2. ] .
  ].
^result

]

{ #category : 'Private' }
Repository >> _getNumThreads [
  ^ SessionTemps current at: #GsOverrideNumThreads
      ifAbsent: [ (SystemRepository numberOfExtents) * 2 ]

]

{ #category : 'Private' }
Repository >> _getShrinkRepository [
  SessionTemps current at: #GsShrinkRepository
    ifAbsent: [ ^0 ].
  ^1

]

{ #category : 'Online Backup Support' }
Repository >> failOverStatus [
  | arr str |
  arr := self failOverStatusInfo .
  (arr at: 1) ifNotNil:[:aString | ^ aString ].

  str := 'This repository last checkpoint file ', (arr at: 2) asString .
      str add: ' record '; add: (arr at:3) asString ; lf ;
          add: ' Other repository restored to checkpoint file '; add:(arr at:4) asString;
          add:' record '; add: (arr at: 5) asString .
  ((arr at: 6) > (arr at: 4) or:[
    (arr at: 6) == (arr at:4) and:[ (arr at:7) > (arr at:5)]]) ifTrue:[
       str lf ;
          add: '                  restored to commit file '; add:(arr at:6) asString;
          add:' record '; add:(arr at:7) asString.
  ].
  ^ str

]

{ #category : 'Online Backup Support' }
Repository >> failOverStatusInfo [
  "Returns an Array describing the tranlog position of this repository
   and the restore status of the slave system do which this repository's
   logsender is connected. The Array contains the following elements:

    1: an error String or nil
    2: a SmallInteger, tranlog fileId of last checkpoint written by this stone .
    3: a SmallInteger, tranlog blockId  of last checkpoint written by this stone .

    4: a SmallInteger, tranlog fileId of last checkpoint restored by slave stone .
    5: a SmallInteger, tranlog blockId of last checkpoint restored by slave stone .
    6: a SmallInteger, tranlog fileId of last commit restored by slave stone.
    7: a SmallInteger, tranlog blockId of last commit restored by slave stone.
"
  | rs fs |
  System sessionIsOnStoneHost ifFalse:[
    ^ #( 'this session must be on stone host (on stone shared cache)' 0 0 0 0 ).
  ].
  rs := self restoreStatusInfo .
  (rs at: 2) == 0 ifFalse:[
   ^ { 'failover not supported, this repository is in restore: ', (rs at: 1) asString .
        0 . 0 . 0 . 0 }
  ].
  System logsenderSessionId == 0 ifTrue:[
    ^ #( 'logsender not running on this repository, failOverStatus not available' 0 0 0 0 ).
  ].
  fs :=  System _zeroArgPrim: 80 .
  fs _isArray ifFalse:[ ^ { fs asString . 0 . 0 . 0 . 0 }  ].
  ^ fs

]

{ #category : 'Online Backup Support' }
Repository >> failOverToSlave [
   "For use on the master system of a hotstandby pair.
    Suspends commits, starts a checkpoint,
    and waits for completion of the checkpoint, and puts this
    repository into restore mode.

    After waiting for restore in the slave system to finish restoring the checkpoint
    just written by failOverToSlave , execute
       Repository >> commitRestore
    on the slave system .  Then start an inverted pair of logsender/logreceiver.
    The slave system is now ready to be the master and this system can be
    put into continuousRestoreFromArchiveLogs , using the tranlogs written
    by the logreceiver just started.

    Returns a failover time ( a DateAndTime ).
    "
  | ts |
  ts := System _zeroArgPrim: 140 .
  ^ DateAndTime posixSeconds: ts offset: Duration zero .

]

{ #category : 'Deprecated (Single Ref Path)' }
Repository >> findReferencePathToObject: anObject [

"Finds a reference path to an object.  Returns an instance of GsSingleRefPathResult,
 for which each element represents a connection leading between the search
 objects and the reference in the repository.
 The first element will usually be one of the objects in the limit set, and the last
 element will be the argument.

 The searchObj must be a committed non-special object. This method is not to be used
 for classes or metaclasses. Objects in the stone's list of dead objects are excluded from
 the scan.  It is an error if the object is in the stone's dead objects set.

 Only a single reference path to each object is returned. It is possible for any object
 to be connected to the repository through multiple reference paths.

 It is possible for an object to be disonnected from the repository and therefore have no
 reference path. In this case, the result GsSingleRefPathResult isDead will be true.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData.

 A reference path scan for an object is terminated and considered completed if any class
 object or GsObjectSecurityPolicy is found in the reference path.  Normally all classes are
 connected to the repository and will be included in the limit set.  However, for garbage
 collection purposes, disconnected classes are considered live objects if any other object
 references them.  In other words, a class is a live object if any live instances of that
 class exist in the repository.
"

self deprecated: 'Repository>>findReferencePathToObject: deprecated v3.4. ',
 'Use (GsSingleRefPathFinder newForSearchObjects: {anObject}) runScan buildResultObjects first'.

System needsCommit ifTrue: [ System _error: #rtErrAbortWouldLoseData ] .

^ (GsSingleRefPathFinder newForSearchObjects: { anObject }) runScan
      buildResultObjects first

]

{ #category : 'Deprecated (Single Ref Path)' }
Repository >> findReferencePathToObjects: anArray findAllRefs: findAllRefs printToLog: printToLog [

"Finds a reference path to an array of objects.  Returns an array of GsSingleRefPathResult
 showing how each object is connected to the repository by reference.

 Note that arguments to this method other than <anArray> are not used in v3.4.

 See the comments in findReferencePathToObject: for more information.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData."

self deprecated: 'Repository>>findReferencePathToObjects:findAllRefs:printToLog: deprecated v3.4. ',
 'Use (GsSingleRefPathFinder newForSearchObjects: anArray) runScan buildResultObjects'.

System needsCommit ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

^ (GsSingleRefPathFinder newForSearchObjects: anArray) runScan
      buildResultObjects

]

{ #category : 'Deprecated (Single Ref Path)' }
Repository >> findReferencePathToObjs: anArray limitObjArray: limitArray findAllRefs: findAllRefs printToLog: printToLog [

"Finds a reference path to an array of objects.  Returns an array of GsSingleRefPathResult
 showing how each object is connected to the repository by reference.

 Note that arguments to this method other than <anArray> are not used in v3.4.

 See the comments in findReferencePathToObject: for more information.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData."

self deprecated: 'Repository>>findReferencePathToObjs:limitObjArray:findAllRefs:printToLog: deprecated v3.4. ',
 'Use (GsSingleRefPathFinder newForSearchObjects: anArray) runScan buildResultObjects'.

System needsCommit ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

^ (GsSingleRefPathFinder newForSearchObjects: anArray) runScan
      buildResultObjects

]

{ #category : 'Default numThreads' }
Repository >> getDefaultNumThreads [

"Returns the default number of threads to use for multi threaded operations"

 ^ SessionTemps current at: #GsOverrideNumThreads
     ifAbsent: [ 2 ]

]

{ #category : 'Backup and Restore' }
Repository >> lastFailoverTime [

"If system is in restore from transaction logs, returns the
 last failOver timestamp detected while reading transaction logs,
 or nil if no failOver timestamp has been processed since restore started.
 If system is not in restore, returns the time from the most
 recent suspendCommitsForFailover, or nil if no suspendCommitsForFailover
 has been executed.

 Result is a DateAndTime or nil."

 | info tm suspendTime |
 ((info := self restoreStatusInfo) at: 2) == 2 ifTrue:[
   (tm := info at: 13) == 0 ifTrue:[ ^ nil ].
   ^ DateAndTime posixSeconds: tm offset: Duration zero
 ].
 (suspendTime := info at: 14) == 0 ifTrue:[ ^ nil ].
 ^ DateAndTime posixSeconds: suspendTime offset: Duration zero

]

{ #category : 'Repository Analysis' }
Repository >> loadGcCandidatesFromFile: aString intoHiddenSet: hiddenSetId [

"Given a file which is binary list of dead objects (produced by one of the
 #findDisconnectedObjects: methods), load the objects into the hidden set
 with the given ID.  NOTE: in 3.4 the hiddenSetId changed from a SmallInteger to
 one of the symbols in GsBitmap>>hiddenSetSpecifiers.

 Only objects which exist and are not in the stone's dead object
 set are added to the hidden set.

 Used by GsObjectInventory to profile garbage in the repository.

 Returns an Array containing 3 SmallIntegers:
  1 - Total number of objects in the file.
  2 - Number of valid objects loaded into the hidden set.
  3 - Number of invalid objects not loaded.

 Requires the GarbageCollection privilege.

 Signals an error if hiddenSetId specifies a hidden set only writable
 by SystemUser and the session is not logged in as SystemUser.
"

| gsBitmap numInvalid result bmSize |

gsBitmap := GsBitmap newForHiddenSet: hiddenSetId.
numInvalid := gsBitmap readFromFile: aString.
bmSize := gsBitmap size.
result := Array with: (bmSize + numInvalid)
                with: bmSize
                with: numInvalid.
^result

]

{ #category : 'Default numThreads' }
Repository >> setDefaultNumThreads: numThreads [

"Sets default number of threads to use for multi threaded operations"

 SessionTemps current at: #GsOverrideNumThreads put: numThreads

]

{ #category : 'Default numThreads' }
Repository >> setDefaultNumThreads: numThreads during: aBlock [
"Sets default number of threads to use for the execution of aBlock containing 
 a multi threaded operation and then resets it after the operation completes."

| prevDefault |
 prevDefault := SessionTemps current at: #GsOverrideNumThreads otherwise: nil.
 SessionTemps current at: #GsOverrideNumThreads put: numThreads.
 ^ aBlock ensure:
     [prevDefault 
        ifNil: [ SessionTemps current removeKey: #GsOverrideNumThreads ]
        ifNotNil: [  SessionTemps current at: #GsOverrideNumThreads put: prevDefault ]
     ].

]
